CREATE OR REPLACE PACKAGE PKG_CLOB
IS
  -- populate the CLOB/NCLOB column --
  FUNCTION select_clob  ( PC$Clause IN VARCHAR2 ) RETURN BOOLEAN ;
  FUNCTION select_nclob ( PC$Clause IN VARCHAR2 ) RETURN BOOLEAN ;
  -- Get and return chunks --
  FUNCTION Get_Chunk RETURN  VARCHAR2 ;
  FUNCTION Get_NChunk RETURN VARCHAR2 ;
  -- Set chunks --
  PROCEDURE Init_Transfer ( PC$Type IN VARCHAR2 ) ;
  FUNCTION  Transfer( PC$Clause IN VARCHAR2, PC$Type IN VARCHAR2 ) RETURN VARCHAR2;
  PROCEDURE Set_Chunk ( PC$Chunk IN VARCHAR2 )  ;
  PROCEDURE Set_NChunk ( PC$Chunk IN VARCHAR2 ) ;
  
  PROCEDURE Set_ChunkSize ( PN$Size IN PLS_INTEGER ) ;
END ;
/


CREATE OR REPLACE PACKAGE BODY PKG_CLOB 
IS
 
  GL$Clob       CLOB ;                -- global CLOB variable
  GL$NClob      NCLOB ;               -- global NCLOB variable  
  GN$Pos        PLS_INTEGER := 1 ;    -- global current pos in the CLOB
  GN$ChunkSize  PLS_INTEGER := 8192 ; -- cannot be greater that 32767
  
  ----------------------------------------
  -- Get the content of the CLOB column --
  ----------------------------------------
  FUNCTION select_clob ( PC$Clause IN VARCHAR2 ) RETURN BOOLEAN
  IS
  BEGIN
    execute immediate PC$Clause into GL$Clob ;
    GN$Pos := 1 ;
    RETURN TRUE ;
  EXCEPTION
    WHEN OTHERS THEN
	  RETURN FALSE ;
  END select_clob ;
 
  -----------------------------------------
  -- Get the content of the NCLOB column --
  -----------------------------------------
  FUNCTION select_nclob ( PC$Clause IN VARCHAR2 ) RETURN BOOLEAN
  IS
  BEGIN
    execute immediate PC$Clause into GL$NClob ;
    GN$Pos := 1 ;
    RETURN TRUE ;
  EXCEPTION
    WHEN OTHERS THEN
	  RETURN FALSE ;
  END select_nclob ;
 
  ------------------------
  --  get a CLOB chunk  --
  ------------------------
  FUNCTION Get_Chunk RETURN VARCHAR2
  IS
    LC$Chunk  VARCHAR2(32767) ;
    LN$pos    PLS_INTEGER := 1 ;
  BEGIN
	LC$Chunk := SUBSTR( GL$Clob, GN$Pos, GN$ChunkSize ) ;
	GN$Pos := GN$Pos + GN$ChunkSize ;
    RETURN LC$Chunk ;
  EXCEPTION
    WHEN OTHERS THEN
	  RETURN NULL ;
  END Get_Chunk ;  
 
  -------------------------
  --  get a NCLOB chunk  --
  -------------------------
  FUNCTION Get_NChunk RETURN VARCHAR2
  IS
    LC$Chunk  VARCHAR2(32767) ;
	LN$pos    PLS_INTEGER := 1 ;
  BEGIN
	LC$Chunk := SUBSTR( GL$NClob, GN$Pos, GN$ChunkSize ) ;
	GN$Pos := GN$Pos + GN$ChunkSize ;
    RETURN LC$Chunk ;
  EXCEPTION
    WHEN OTHERS THEN
	  RETURN NULL ;
  END Get_NChunk ;
  
  --------------------------
  --  set the chunk size  --
  --------------------------
  PROCEDURE Set_ChunkSize ( PN$Size IN PLS_INTEGER )
  IS
  Begin
    If Nvl(PN$Size,0) between 128 and 32767 Then
      GN$ChunkSize := PN$Size  ;
    End if ;
  End Set_ChunkSize;
  

  ----------------------------------
  --  set the chunks to populate  --
  --  the CLOB column             --
  ----------------------------------
  PROCEDURE Set_Chunk ( PC$Chunk IN VARCHAR2 )
  Is
  Begin
    If PC$Chunk Is Not Null Then
      GL$Clob := GL$Clob || PC$Chunk ;  
    End if ;
  End Set_Chunk;

  ----------------------------------
  --  set the chunks to populate  --
  --  the NCLOB column             --
  ----------------------------------
  PROCEDURE Set_NChunk ( PC$Chunk IN VARCHAR2 )
  IS
    NContent NVARCHAR2(32767) := PC$Chunk;
  BEGIN
    IF PC$Chunk IS NOT NULL THEN
      GL$NClob := GL$NClob || NContent ;
    END IF ;
  END Set_NChunk;
  
  PROCEDURE Init_Transfer ( PC$Type IN VARCHAR2 )
  Is
  Begin
    If Upper(PC$Type) = 'CLOB' Then
      GL$Clob := '' ;
    ElsIf Upper(PC$Type) = 'NCLOB' Then
      GL$NClob := '' ;
    End if ;
  End Init_Transfer;
  
  
  FUNCTION  Transfer( PC$Clause IN VARCHAR2, PC$Type IN VARCHAR2 )
  RETURN VARCHAR2
  Is
  Begin
    If Upper(PC$Type) = 'CLOB' Then
      execute immediate PC$Clause using GL$Clob ;
    Else
      execute immediate PC$Clause using GL$NClob ;
    End if ;
    return 'OK' ;
  Exception
    When Others Then
      return SQLERRM ;
  End Transfer ;
  
  
END ;
/
 